@@ -1,11 +1,12 @@
 """AMQP Connections."""
 # Copyright (C) 2007-2008 Barry Pederson <bp@barryp.org>
-from __future__ import absolute_import, unicode_literals
 
 import logging
 import socket
 import uuid
 import warnings
+from array import array
+from time import monotonic
 
 from vine import ensure_promise
 
@@ -16,7 +17,6 @@ from .exceptions import (AMQPDeprecationWarning, ChannelError, ConnectionError,
                          ConnectionForced, RecoverableChannelError,
                          RecoverableConnectionError, ResourceError,
                          error_for_code)
-from .five import array, items, monotonic, range, string, values
 from .method_framing import frame_handler, frame_writer
 from .transport import Transport
 
@@ -39,9 +39,12 @@ START_DEBUG_FMT = """
 Start from server, version: %d.%d, properties: %s, mechanisms: %s, locales: %s
 """.strip()
 
-__all__ = ['Connection']
+__all__ = ('Connection',)
 
 AMQP_LOGGER = logging.getLogger('amqp')
+AMQP_HEARTBEAT_LOGGER = logging.getLogger(
+    'amqp.connection.Connection.heartbeat_tick'
+)
 
 #: Default map for :attr:`Connection.library_properties`
 LIBRARY_PROPERTIES = {
@@ -91,16 +94,17 @@ class Connection(AbstractChannel):
     client name. For EXTERNAL authentication both userid and password are
     ignored.
 
-    The 'ssl' parameter may be simply True/False, or for Python >= 2.6
-    a dictionary of options to pass to ssl.wrap_socket() such as
-    requiring certain certificates.
+    The 'ssl' parameter may be simply True/False, or
+    a dictionary of options to pass to :class:`ssl.SSLContext` such as
+    requiring certain certificates. For details, refer ``ssl`` parameter of
+    :class:`~amqp.transport.SSLTransport`.
 
     The "socket_settings" parameter is a dictionary defining tcp
     settings which will be applied as socket options.
 
     When "confirm_publish" is set to True, the channel is put to
     confirm mode. In this mode, each published message is
-    confirmed using Publisher confirms RabbitMQ extention.
+    confirmed using Publisher confirms RabbitMQ extension.
     """
 
     Channel = Channel
@@ -241,7 +245,7 @@ class Connection(AbstractChannel):
 
         self.channels = {}
         # The connection object itself is treated as channel 0
-        super(Connection, self).__init__(self, 0)
+        super().__init__(self, 0)
 
         self._frame_writer = None
         self._on_inbound_frame = None
@@ -263,7 +267,7 @@ class Connection(AbstractChannel):
         self.on_unblocked = on_unblocked
         self.on_open = ensure_promise(on_open)
 
-        self._avail_channel_ids = array('H', range(self.channel_max, 0, -1))
+        self._used_channel_ids = array('H')
 
         # Properties set in the Start method
         self.version_major = 0
@@ -274,6 +278,14 @@ class Connection(AbstractChannel):
 
         self.connect_timeout = connect_timeout
 
+    def __repr__(self):
+        if self._transport:
+            return f'<AMQP Connection: {self.host}/{self.virtual_host} '\
+                   f'using {self._transport} at {id(self):#x}>'
+        else:
+            return f'<AMQP Connection: {self.host}/{self.virtual_host} '\
+                   f'(disconnected) at {id(self):#x}>'
+
     def __enter__(self):
         self.connect()
         return self
@@ -316,7 +328,7 @@ class Connection(AbstractChannel):
             while not self._handshake_complete:
                 self.drain_events(timeout=self.connect_timeout)
 
-        except (OSError, IOError, SSLError):
+        except (OSError, SSLError):
             self.collect()
             raise
 
@@ -363,7 +375,7 @@ class Connection(AbstractChannel):
         self.version_major = version_major
         self.version_minor = version_minor
         self.server_properties = server_properties
-        if isinstance(mechanisms, string):
+        if isinstance(mechanisms, str):
             mechanisms = mechanisms.encode('utf-8')
         self.mechanisms = mechanisms.split(b' ')
         self.locales = locales.split(' ')
@@ -378,7 +390,7 @@ class Connection(AbstractChannel):
         cap = client_properties.setdefault('capabilities', {})
         cap.update({
             wanted_cap: enable_cap
-            for wanted_cap, enable_cap in items(self.negotiate_capabilities)
+            for wanted_cap, enable_cap in self.negotiate_capabilities.items()
             if scap.get(wanted_cap)
         })
         if not cap:
@@ -394,7 +406,7 @@ class Connection(AbstractChannel):
         else:
             raise ConnectionError(
                 "Couldn't find appropriate auth mechanism "
-                "(can offer: {0}; available: {1})".format(
+                "(can offer: {}; available: {})".format(
                     b", ".join(m.mechanism
                                for m in self.authentication
                                if m.mechanism).decode(),
@@ -454,32 +466,37 @@ class Connection(AbstractChannel):
         return self._transport and self._transport.connected
 
     def collect(self):
-        try:
-            if self._transport:
-                self._transport.close()
+        if self._transport:
+            self._transport.close()
 
-            temp_list = [x for x in values(self.channels or {})
-                         if x is not self]
-            for ch in temp_list:
+        if self.channels:
+            # Copy all the channels except self since the channels
+            # dictionary changes during the collection process.
+            channels = [
+                ch for ch in self.channels.values()
+                if ch is not self
+            ]
+
+            for ch in channels:
                 ch.collect()
-        except socket.error:
-            pass  # connection already closed on the other end
-        finally:
-            self._transport = self.connection = self.channels = None
+        self._transport = self.connection = self.channels = None
 
     def _get_free_channel_id(self):
-        try:
-            return self._avail_channel_ids.pop()
-        except IndexError:
-            raise ResourceError(
-                'No free channel ids, current={0}, channel_max={1}'.format(
-                    len(self.channels), self.channel_max), spec.Channel.Open)
+        for channel_id in range(1, self.channel_max):
+            if channel_id not in self._used_channel_ids:
+                self._used_channel_ids.append(channel_id)
+                return channel_id
+
+        raise ResourceError(
+            'No free channel ids, current={}, channel_max={}'.format(
+                len(self.channels), self.channel_max), spec.Channel.Open)
 
     def _claim_channel_id(self, channel_id):
-        try:
-            return self._avail_channel_ids.remove(channel_id)
-        except ValueError:
-            raise ConnectionError('Channel %r already open' % (channel_id,))
+        if channel_id in self._used_channel_ids:
+            raise ConnectionError(f'Channel {channel_id!r} already open')
+        else:
+            self._used_channel_ids.append(channel_id)
+            return channel_id
 
     def channel(self, channel_id=None, callback=None):
         """Create new channel.
@@ -583,7 +600,7 @@ class Connection(AbstractChannel):
                 (reply_code, reply_text, method_sig[0], method_sig[1]),
                 wait=spec.Connection.CloseOk,
             )
-        except (OSError, IOError, SSLError):
+        except (OSError, SSLError):
             # close connection
             self.collect()
             raise
@@ -705,8 +722,8 @@ class Connection(AbstractChannel):
         Keyword Arguments:
             rate (int): Previously used, but ignored now.
         """
-        AMQP_LOGGER.debug('heartbeat_tick : for connection %s',
-                          self._connection_id)
+        AMQP_HEARTBEAT_LOGGER.debug('heartbeat_tick : for connection %s',
+                                    self._connection_id)
         if not self.heartbeat:
             return
 
@@ -719,7 +736,7 @@ class Connection(AbstractChannel):
             self.last_heartbeat_received = monotonic()
 
         now = monotonic()
-        AMQP_LOGGER.debug(
+        AMQP_HEARTBEAT_LOGGER.debug(
             'heartbeat_tick : Prev sent/recv: %s/%s, '
             'now - %s/%s, monotonic - %s, '
             'last_heartbeat_sent - %s, heartbeat int. - %s '
@@ -735,7 +752,7 @@ class Connection(AbstractChannel):
 
         # send a heartbeat if it's time to do so
         if now > self.last_heartbeat_sent + self.heartbeat:
-            AMQP_LOGGER.debug(
+            AMQP_HEARTBEAT_LOGGER.debug(
                 'heartbeat_tick: sending heartbeat for connection %s',
                 self._connection_id)
             self.send_heartbeat()
@@ -743,9 +760,10 @@ class Connection(AbstractChannel):
 
         # if we've missed two intervals' heartbeats, fail; this gives the
         # server enough time to send heartbeats a little late
-        if (self.last_heartbeat_received and
-                self.last_heartbeat_received + 2 *
-                self.heartbeat < monotonic()):
+        two_heartbeats = 2 * self.heartbeat
+        two_heartbeats_interval = self.last_heartbeat_received + two_heartbeats
+        heartbeats_missed = two_heartbeats_interval < monotonic()
+        if self.last_heartbeat_received and heartbeats_missed:
             raise ConnectionForced('Too many heartbeats missed')
 
     @property
