@@ -1,15 +1,13 @@
-from __future__ import absolute_import, unicode_literals
+from unittest.mock import Mock, patch, sentinel
 
 import pytest
 from vine import promise
 
 from amqp import spec
-from amqp.abstract_channel import (
-    AbstractChannel, IGNORED_METHOD_DURING_CHANNEL_CLOSE
-)
+from amqp.abstract_channel import (IGNORED_METHOD_DURING_CHANNEL_CLOSE,
+                                   AbstractChannel)
 from amqp.exceptions import AMQPNotImplementedError, RecoverableConnectionError
 from amqp.serialization import dumps
-from case import Mock, patch, sentinel
 
 
 class test_AbstractChannel:
@@ -148,7 +146,9 @@ class test_AbstractChannel:
             )
             if method in (spec.Channel.Close, spec.Channel.CloseOk):
                 self.c._METHODS.__getitem__.assert_called_once_with(method)
-                self.c._callbacks[method].assert_called_once()
+                self.c._callbacks[method].assert_called_once_with(
+                    sentinel.CONTENT
+                )
             else:
                 self.c._METHODS.__getitem__.assert_not_called()
                 self.c._callbacks[method].assert_not_called()
